/*
 * Tapidbus library
 * Copyright (C) 2005 INdT 
 * @author Luiz Augusto von Dentz <luiz.dentz@indt.org.br>
 * @author Andre Moreira Magalhaes <andre.magalhaes@indt.org.br>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef __TAPIDBUS_H__
#define __TAPIDBUS_H__

#include <glib.h>
#include <dbus/dbus.h>

G_BEGIN_DECLS

typedef struct _TapiDBus       TapiDBus;
typedef struct _TapiDBusMethod TapiDBusMethod;
typedef struct _TapiDBusSignal TapiDBusSignal;

typedef DBusMessage *(TapiDBusMethodHandlerFunc)         (TapiDBus    *td, 
                                                          DBusMessage *message, 
                                                          gpointer     user_data);
typedef void         (TapiDBusSignalHandlerFunc)         (TapiDBus    *td, 
                                                          DBusMessage *message, 
                                                          gpointer     user_data);
typedef void         (TapiDBusPendingCallNotifyFunction) (TapiDBus    *td,
                                                          DBusMessage *message,
                                                          gpointer     user_data);

struct _TapiDBus {
    DBusConnection            *conn;         /* Connection related to the interface */
    gchar                     *service_name; /* Service name */
    gchar                     *path;         /* Path name */
    TapiDBusMethod            *methods;      /* Methods */
    TapiDBusSignal            *signals;      /* Signals */
};

struct _TapiDBusMethod {
    gchar                     *iface;         /* Interface name */
    gchar                     *name;          /* Method name */
    gchar                     *signature;     /* Signature */
    TapiDBusMethodHandlerFunc *handle_func;
    gpointer                   user_data;
};

struct _TapiDBusSignal {
    gchar                     *iface;         /* Interface name */
    gchar                     *name;          /* Signal name */
    gchar                     *signature;     /* Signature */
    TapiDBusSignalHandlerFunc *handle_func;
    gpointer                   user_data;
};

TapiDBus           *tapidbus_new                    (void);
TapiDBus           *tapidbus_new_with_service       (const gchar                       *service_name,
                                                     const gchar                       *path,
                                                     TapiDBusMethod                    *methods,
                                                     TapiDBusSignal                    *signals);
gboolean            tapidbus_set_service            (TapiDBus                          *td,
                                                     const gchar                       *service_name,
                                                     const gchar                       *path,
                                                     TapiDBusMethod                    *methods,
                                                     TapiDBusSignal                    *signals);
void                tapidbus_destroy                (TapiDBus                          *td);

gboolean            tapidbus_send                   (TapiDBus                          *td,
                                                     const gchar                       *destination,
                                                     const gchar                       *path, 
                                                     const gchar                       *iface, 
                                                     const gchar                       *method,
                                                     gint                               type, 
                                                     ...);
DBusPendingCall    *tapidbus_send_with_reply        (TapiDBus                          *td,
                                                     const gchar                       *destination,
                                                     const gchar                       *path,
                                                     const gchar                       *iface,
                                                     const gchar                       *method,
                                                     TapiDBusPendingCallNotifyFunction  function,
                                                     gpointer                           user_data,
                                                     gint                               timeout_milliseconds,
                                                     gint                               type,
                                                     ...);
DBusMessage        *tapidbus_send_with_reply_block  (TapiDBus                          *td,
                                                     const gchar                       *destination,
                                                     const gchar                       *path,
                                                     const gchar                       *iface,
                                                     const gchar                       *method,
                                                     gint                               timeout_milliseconds,
                                                     gint                               type, 
                                                     ...);
gboolean            tapidbus_emit_signal            (TapiDBus                          *td,
                                                     const gchar                       *path,
                                                     const gchar                       *iface,
                                                     const gchar                       *method,
                                                     gint                               type, 
                                                     ...);
DBusMessage        *tapidbus_build_method_return    (DBusMessage                       *in_reply_to,
                                                     gint                               type,
                                                     ...);

gboolean            tapidbus_get_int32              (DBusMessage                       *message,
                                                     gint                              *value);
gboolean            tapidbus_get_string             (DBusMessage                       *message,
                                                     gchar                            **value);
gboolean            tapidbus_get_boolean            (DBusMessage                       *message,
                                                     gboolean                          *value);
gboolean            tapidbus_get_double             (DBusMessage                       *message,
                                                     gdouble                           *value);
gboolean            tapidbus_get_args               (DBusMessage                       *message,
                                                     gint                               type,
                                                     ...);

G_END_DECLS

#endif

